"""
CMDC SDK 异常类型。
"""

from __future__ import annotations


class CMDCError(Exception):
    """SDK 基础异常。"""

    def __init__(self, message: str, status_code: int | None = None) -> None:
        super().__init__(message)
        self.message = message
        self.status_code = status_code


class AuthError(CMDCError):
    """API Key 无效或缺失（401）。"""


class SessionNotFoundError(CMDCError):
    """Session 不存在或 Agent 进程已终止（404）。"""


class RateLimitError(CMDCError):
    """超出请求频率限制（429）。"""

    def __init__(self, message: str, retry_after: int | None = None) -> None:
        super().__init__(message, status_code=429)
        self.retry_after = retry_after


class SessionCreateError(CMDCError):
    """Agent 创建失败（422）。"""


class BadRequestError(CMDCError):
    """请求参数错误（400）。"""
